/*
 *          Copyright (c) mjh-EDV Beratung, 1996-1998
 *     mjh-EDV Beratung - 63263 Neu-Isenburg - Rosenstrasse 12
 *          Tel +49 6102 328279 - Fax +49 6102 328278
 *                Email info@mjh.teddy-net.com
 *
 *    Author: Jordan Hrycaj <jordan@mjh.teddy-net.com>
 *
 *   $Id: plain.c,v 1.5 1999/01/09 18:36:14 jordan Exp $
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 */

#include "common-stuff.h"
#include "plain.h"

/* ---------------------------------------------------------------------------- *
 *                   dummy ciphers for testing                                  *
 * ---------------------------------------------------------------------------- */

#ifdef CIPHER_PLAIN

static void
plain8 (d, out, in)
     void   *d;
     byte *out;
     byte  *in;
{
  memcpy (out, in, 8);
}

static void
plain16 (d, out, in)
     void   *d;
     byte *out;
     byte  *in;
{
  memcpy (out, in, 16);
}

static void
invert8 (d, out, in)
     void   *d;
     byte *out;
     byte  *in;
{
  memcpy (out, in, 8);
  *(u32*) out    ^= (u32)0 ;
  *(u32*)(out+4) ^= (u32)0 ;
}


static void
invert16 (d, out, in)
     void   *d;
     byte *out;
     byte  *in;
{
  memcpy (out, in, 16);
  *(u32*) out       ^= (u32)0 ;
  *(u32*)(out +  4) ^= (u32)0 ;
  *(u32*)(out +  8) ^= (u32)0 ;
  *(u32*)(out + 12) ^= (u32)0 ;
}


static int
set_key (d, buf, len)
     void       *d;
     byte     *buf;
     unsigned  len;
{
  ((char*) d) [0] = 3 ;
  ((char*) d) [1] = 2 ;
  ((char*) d) [2] = 1 ;
  return 0;
}
#endif /* CIPHER_PLAIN */


/* ---------------------------------------------------------------------------- *
 *                   dummy crc for testing                                      *
 * ---------------------------------------------------------------------------- */

#ifdef CRC16_FRAME

/* The generator polynomial for crc16, used only to verify the keys 
   stored on disk in the key file.  This algorithm is used solely
   to detect a corrupt line entry */ 
#define CRC16_POLYNOMIAL 0xA001 
#define CRC16_PRESET     0xffff

static void
crc16_init (void *Accu)
{
  memset (Accu, 0, 5 * sizeof (unsigned));
  *(unsigned*)Accu =  CRC16_PRESET ;
}

static void
crc16_final (void *Accu)
{
}

static byte *
crc16_read (void *Accu)
{
  unsigned accu = *(unsigned *)Accu ;
  unsigned    n = sizeof (unsigned) ;
  while (n --) {
    /* store in big endian */
    ((byte *)Accu)[n] = accu & 0xff ;
    accu >>= 8 ;
  }
  memcpy (Accu +     sizeof (unsigned), Accu,     sizeof (unsigned));
  memcpy (Accu + 2 * sizeof (unsigned), Accu,     sizeof (unsigned));
  memcpy (Accu + 3 * sizeof (unsigned), Accu, 2 * sizeof (unsigned));
  return Accu;
}

static void
crc16_write (Accu, buf, len)
     void *Accu;
     byte  *buf;
     size_t len;
{
  unsigned accu = *(unsigned *)Accu ;
  unsigned i = -1, j; 
  
  /* note that this algorithm shifts right although I find
     it more natural to shift left - it is used that way
     in some INDACTIC implementation, for example */

  while (++ i < len) { 
    
       /* xor the next byte from input (remember: x == x^0) */ 
       accu ^= (* buf ++ & 0x00ff);

       j = 8;
       while (j --) {
	 /* shift right through CARRY, 
	    xor the polynomial if the CARRY is set */
	 if (accu & 0x0001) { 
	   accu >>= 1; 
	   accu  ^= CRC16_POLYNOMIAL; 
           continue ; 
	 }
         accu >>= 1; 
       } 
  } 
  *(unsigned *)Accu = accu ;
} 
#endif /* CRC16_FRAME */

/* ---------------------------------------------------------------------------- *
 *             dummy pseudo random generators for testing                       *
 * ---------------------------------------------------------------------------- */

#ifdef LINEAR8_RANDOM
#include "randomgen.h"
#include "peks-baseXX.h"
#include "baseXX.h"
#include "peks-internal.h"

typedef struct { mpz_t s, a, b, M ; } linear_state ;

static int
linear_seed8 (linear_state *state, byte *key)
{
  char     *t = bin2base64 (key, 8) ;
  unsigned ok = base64toMpz (& state->s, t) ;
  XFREE (t) ;
  return ok ? 0 : -1 ;
}


static int		/* key := (M, a, b) */
linear_init8 (linear_state *state, byte *key)
{
  char sv, *t = bin2base64 (key, 24) ;

  mpz_init (state->s) ;
  mpz_init (state->a) ;
  mpz_init (state->b) ;
  mpz_init (state->M) ;

  /* M := t [0 .. 7] */
  if (t [0] == 0) {		/* do not let the modulus become too small */
    t [0] = 0x80 ;
    if (t [1] == 0)
      t [1] = 0x01 ;
  }
  sv = t [8] ; t [8] = '\0' ;
  if (base64toMpz (& state->M, t) == 0)
    goto error ;


  /* a := t [8 .. 15] */
  if ((t [8] = sv) == 0)	/* do not let a become too small */
    t [8] = 0x80 ;
  sv = t [16] ;  t [16] = '\0' ;
  if (base64toMpz (& state->a, t + 8) == 0)
    goto error ;


  /* b := t [16 .. 24] */
  t [16] = sv ;
  if (base64toMpz (& state->b, t + 16) == 0)
    goto error ;


  XFREE (t) ;
  return 0;
  /*@NOTREACHED@*/

 error:
  mpz_clear (state->s) ;
  mpz_clear (state->a) ;
  mpz_clear (state->b) ;
  mpz_clear (state->M) ;
  XFREE (t) ;
  return -1;
}


static void
linear_destroy (linear_state *state)
{
  if (state->s == 0)
    mpz_clear (state->s) ;
  if (state->a == 0)
    mpz_clear (state->a) ;
  if (state->b == 0)
    mpz_clear (state->b) ;
  if (state->M == 0)
    mpz_clear (state->M) ;
}


static void
linear_random8 (linear_state *state, byte *outbuf)
{
  char *t, *u ;
  unsigned n ;

  /* s := a*s + b (mod M) */
  mpz_mul (state->s, state->a, state->s) ;
  mpz_mod (state->s, state->s, state->M) ;
  mpz_add (state->s, state->s, state->b) ;
  mpz_mod (state->s, state->s, state->M) ;

  u = base64toBin (t = mpz2base64 (&state->s), &n) ;
  XFREE (t) ;

  if (n > 8)
    n = 8 ;
  memcpy (outbuf, u, n) ;
  XFREE (u) ;

  if (n < 8)
    memset (outbuf + n, 0, 8 - n);
}

#endif /* LINEAR8_RANDOM */

/* ---------------------------------------------------------------------------- *
 *                   interfaces                                                 *
 * ---------------------------------------------------------------------------- */

#ifdef CIPHER_PLAIN
const char *
plain_get_info (int algo, size_t *keylen,
		size_t *blocksize, size_t *contextsize,
		int	(**r_setkey)( void *c, byte *key, unsigned keylen ),
		void (**r_encrypt)( void *c, byte *outbuf, byte *inbuf ),
		void (**r_decrypt)( void *c, byte *outbuf, byte *inbuf )
		)
{
  switch (algo) {
  case 8:
    *keylen      =     128 ;
    *blocksize   =       8 ;
    *contextsize =       3 ;
    *r_setkey    = &set_key ;
    *r_encrypt   = &plain8 ;
    *r_decrypt   = &plain8 ;
    return "PLAIN8" ;

  case -8:
    *keylen      =      128 ;
    *blocksize   =        8 ;
    *contextsize =        3 ;
    *r_setkey    =  &set_key ;
    *r_encrypt   = &invert8 ;
    *r_decrypt   = &invert8 ;
    return "INVERT8" ;

  case 16:
    *keylen      =      128 ;
    *blocksize   =       16 ;
    *contextsize =        3 ;
    *r_setkey    =  &set_key ;
    *r_encrypt   = &plain16 ;
    *r_decrypt   = &plain16 ;
    return "PLAIN16" ;

  case -16:
    *keylen      =       128 ;
    *blocksize   =        16 ;
    *contextsize =         3 ;
    *r_setkey    =   &set_key ;
    *r_encrypt   = &invert16 ;
    *r_decrypt   = &invert16 ;
    return "INVERT16" ;
  }

  return NULL;
}
#endif /* CIPHER_PLAIN */


#ifdef CRC16_FRAME
const char *
crc16_get_info (int algo, size_t *contextsize,
	     byte **r_asnoid, int *r_asnlen, int *r_mdlen,
	     void (**r_init) (void *),
	     void (**r_write)(void *, byte *buf, size_t nbytes),
	     void (**r_final)(void *),
	     byte *(**r_read)(void *)
	     )
{
  *contextsize = 5 * sizeof (unsigned);
  *r_asnoid    = 0;
  *r_asnlen    = 0 ;
  *r_mdlen     = *contextsize;
  *r_init      = crc16_init;
  *r_write     = crc16_write;
  *r_final     = crc16_final;
  *r_read      = crc16_read;
  return "CRC16";
}
#endif /* CRC16_FRAME */


#ifdef LINEAR8_RANDOM
const char *
linear_get_info (int algo, 
		 size_t *initlen,
		 size_t *seedlen,
		 size_t *blocksize,
		 size_t *contextsize,
		 int  (**r_init)    (void *c, byte *key),
		 int  (**r_seed)    (void *c, byte *key),
		 void (**r_random)  (void *c, byte *outbuf),
		 void (**r_destroy) (void *c))
{
  *initlen     = 24 ;
  *seedlen     =  8 ;
  *blocksize   =  8 ;
  *contextsize = sizeof (linear_state) ;
  *r_init      = (int  (*)(void *, byte *))linear_init8 ;
  *r_seed      = (int  (*)(void *, byte *))linear_seed8 ;
  *r_random    = (void (*)(void *, byte *))linear_random8 ;
  *r_destroy   = (void (*)(void *))        linear_destroy ;
  return "lin8" ;
}
#endif /* LINEAR8_RANDOM */
